#include <Arduino.h>

typedef struct NTPtime {
  unsigned long sec;
  unsigned long frac;
};

typedef struct NTPpacket {
  char Version;                   // [0,7]. Leap and Version of the NTP protocol.
  char Stratum;                   // [0,255]. Mode ans Stratum of time server in the server tree.
  char Poll;                    // [-128,127]. The maximum interval between successive messages, in log2 seconds.
  char Precision;               // [-128,127]. The precision of the system clock, in log2 seconds.
  unsigned long RootDelay;      // [0,2^32-1]. Total round trip delay to the reference clock in seconds times 2^16.
  unsigned long RootDispersion; // [0,2^32-1]. Total dispersion to the reference clock, in seconds times 2^16.
  unsigned long ReferenceID;    // ID code of reference clock [0,2^32-1].
  NTPtime Tref;   // Most recent timestamp from the reference clock.
  NTPtime Torig;  // Local time when request was sent from local host.
  NTPtime Trx;    // Local time (on server) that request arrived at server host.
  NTPtime Ttx;    // Local time (on server) that request departed server host.
};

/* reorder bytes in place with PC endianess */
void deendian(NTPtime *t);
/* Computes the sum of fist two times, and stores in third parameter */
bool NTPsum(NTPtime *t0, NTPtime *t1, NTPtime *s);
int NTPdiff(NTPtime *t0, NTPtime *t1, NTPtime *d);
//long NTPdiff2ms(NTPtime *t0, NTPtime *t1); 
void ms2NTP(NTPtime *dest, unsigned long ms);
unsigned long NTP2ms(NTPtime *ntp);
int printNTP(NTPtime *t, const char *name, int prec);
